package com.example.civilizedcity.service.impl;

import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.*;
import com.example.civilizedcity.mapper.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.example.civilizedcity.service.TaCheckService;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 模拟测评;(ta_mock_check)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2022-12-12
 */
@Service
public class TaCheckServiceImpl extends BaseServiceImpl<TaCheckMapper, TaCheck> implements TaCheckService {

    @Autowired
    TaCheckItemMapper taCheckItemMapper;

    @Autowired
    TaCheckItemQuMapper taCheckItemQuMapper;

    @Autowired
    TaCheckAnswerMapper taCheckAnswerMapper;

    @Autowired
    TaCheckAnswerItemMapper taCheckAnswerItemMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TaCheck calcScore(TaCheck taCheck) {

        // 调查问卷分数
        double surveyScore = calcSurveyScore(taCheck);

        // 计算点位分数
        double locScore = calcLocScore(taCheck);

        taCheck.setLocScore(locScore);
        taCheck.setSurveyScore(surveyScore);

        updateById(taCheck);

        return taCheck;
    }

    /**
     * 计算点位分数
     * 计算的前提是, 每份试卷（ta_check_answer）的分数已经计算好了
     * @param taCheck
     * @return
     */
    private double calcLocScore(TaCheck taCheck) {
        List<TaCheckItem> checkItemList = taCheckItemMapper.getListBy(taCheck.getCheckId(), Constants.CHECK_OF_LOC);
        if (null == checkItemList || checkItemList.size() == 0) return 0.0;

        // 所有问卷得分的和
        double totalScore = 0.0;
        for (TaCheckItem checkItem : checkItemList) {
            Double score = taCheckAnswerMapper.sumScore(checkItem.getItemId(), checkItem.getNum());
            double s = null == score ? 0.0 : score;
            checkItem.setScore(s);
            taCheckItemMapper.updateById(checkItem);

            totalScore += s;
        }

        return totalScore;
    }

    /**
     * 计算调查问卷分数
     * 计算的前提是, 每个题目（ta_check_answer_item）的分数已经计算好了
     * 暂仅支持选择题算法
     * 总分数 = 100 / 问卷分数 / 每份问卷的题数
     * @param taCheck
     * @return
     */
    private double calcSurveyScore(TaCheck taCheck) {
        // 目前的需求, 此处应该只能查询到一条数据
        List<TaCheckItem> checkItemList = taCheckItemMapper.getListBy(taCheck.getCheckId(), Constants.CHECK_OF_SURVEY);
        if (null == checkItemList || checkItemList.size() == 0) return 0.0;

        // 所有问卷得分的和
        double totalScore = 0.0;
        for (TaCheckItem checkItem : checkItemList) {
            // 问卷份数
            List<TaCheckAnswer> answerList = taCheckAnswerMapper.getListBy(checkItem.getItemId());
            if (answerList == null || answerList.size() == 0) continue;
            int copyNum = answerList.size();

            // 题目数
            List<TaCheckItemQu> quList = taCheckItemQuMapper.getListBy(checkItem.getItemId());
            if (quList == null || quList.size() == 0) continue;
            int quNum = quList.size();

            // 每题的最大分数
            double maxStandScore = 100.00 / copyNum / quNum;

            // 总分
            double score = 0.0;
            for (TaCheckAnswer answer : answerList) {
                // 获取答案列表
                List<TaCheckAnswerItem> answerItemList = taCheckAnswerItemMapper.getListBy(answer.getAnswerId());
                if (null == answerItemList || answerItemList.size() == 0) continue;

                // 每题得分
                double quScore = 0.0;
                for (TaCheckAnswerItem answerItem : answerItemList) {
                    if (null == answerItem.getScore() || isZero(answerItem.getScore())) {
                        // 如果得分是 0
                        continue;
                    }

                    // 查询对应的问题
                    TaCheckItemQu qu = getQu(quList, answerItem.getQuId());
                    if (null == qu) continue;   // 找不到对应题目不计算
                    if (null == qu.getMaxScore() || isZero(qu.getMaxScore())) continue; // 题目没有维护最大分数, 不计算

                    // 计算转换比
                    double ratio = maxStandScore / qu.getMaxScore();

                    // 当前得分
                    double s = answerItem.getScore();
//                    double s = Math.abs(answerItem.getScore()) > qu.getMaxScore() ?  qu.getMaxScore() : answerItem.getScore();
//                    if (answerItem.getScore() < 0) {
//                        // 在与最大值的比较过程中, 最大值是正数, 有可能改变得分的符号
//                        // 所以此处要纠正过来
//                        s = 0 - Math.abs(s);
//                    }

                    // 分值
                    quScore += s * ratio;
                }

                answer.setScore(quScore);
                taCheckAnswerMapper.updateById(answer);

                // 分值
                score += quScore;
            }

            checkItem.setScore(score);
            taCheckItemMapper.updateById(checkItem);

            totalScore += score;
        }

        return totalScore;
    }

    private TaCheckItemQu getQu(List<TaCheckItemQu> quList, String quId) {
        return quList.stream().filter(q -> q.getQuId().equals(quId)).findFirst().orElse(null);
    }

    private boolean isZero(Double d) {
        return Math.abs(d) < 0.0001;
    }
}